"use strict";

exports.__esModule = true;
exports.default = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _akutils = require("akutils");

var _IsolatedProperty = require("./IsolatedProperty");

var _IsolatedProperty2 = _interopRequireDefault(_IsolatedProperty);

var _reshadow = require("./reshadow");

var _reshadow2 = _interopRequireDefault(_reshadow);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*
	Questions:
	 	* How integrate with time travel debugging
*/

var IsolatedApi = function () {
	function IsolatedApi(store) {
		_classCallCheck(this, IsolatedApi);

		this.store = store;
		this.records = {};
	}

	_createClass(IsolatedApi, [{
		key: "all",
		value: function all(owner) {
			var record = this._record(owner);

			return record.all();
		}
	}, {
		key: "count",
		value: function count(owner) {
			var record = this._record(owner);

			return record.count();
		}
	}, {
		key: "entries",
		value: function entries(owner) {
			var record = this._record(owner);

			return record.entries();
		}
	}, {
		key: "get",
		value: function get(key, owner) {
			var record = this._record(owner);

			return record && record.get(key);
		}
	}, {
		key: "invalidated",
		value: function invalidated(iso) {
			var record = this._record(iso.owner());

			record.invalidated(iso);
		}
	}, {
		key: "obsolete",
		value: function obsolete(iso) {
			var owner = iso.owner();
			var record = this._findRecord(owner);

			record && record.obsolete(iso);
		}
	}, {
		key: "remove",
		value: function remove(key, owner) {
			var record = this._record(owner);

			record && record.remove(key);
		}
	}, {
		key: "removeAllFor",
		value: function removeAllFor(owner) {
			var record = this._record(owner);

			record && record.removeAll();
		}
	}, {
		key: "set",
		value: function set(key, isoType, owner) {
			var record = this._record(owner);

			(0, _akutils.assert)(function (a) {
				return a.is(isoType.getManagedType(), "Managed type required for isolated objects");
			});

			record.set(key, isoType);
		}

		/**
  	Invoked by {@link IsolatedObjectShadowImpl#defineChildProperties} to perform updates based
  	on processed actions.
  		@param {Property} owner - the property managing isolated properties.
  */

	}, {
		key: "update",
		value: function update(owner) {
			var record = this._record(owner);

			record && record.update();
		}
	}, {
		key: "willUnshadow",
		value: function willUnshadow(iso) {
			var record = this._record(iso.owner());

			record.willUnshadow(iso);
		}
	}, {
		key: "_findRecord",
		value: function _findRecord(owner) {
			return this.records[owner.pid()];
		}
	}, {
		key: "_record",
		value: function _record(owner) {
			var records = this.records;
			var rid = owner.dotPath();

			if (!records[rid]) {
				records[rid] = new OwnerRecord(this, owner);
			}

			return records[rid];
		}
	}]);

	return IsolatedApi;
}();

exports.default = IsolatedApi;

var OwnerRecord = function () {
	function OwnerRecord(api, owner) {
		_classCallCheck(this, OwnerRecord);

		this._owner = owner;
		this._invalidated = false;
		this._kv = {};
		this._nextKv = {};
	}

	_createClass(OwnerRecord, [{
		key: "all",
		value: function all() {
			return Object.values(this._kv);
		}
	}, {
		key: "count",
		value: function count() {
			return Object.keys(this._kv).length;
		}
	}, {
		key: "entries",
		value: function entries() {
			return _extends({}, this._kv);
		}
	}, {
		key: "get",
		value: function get(key) {
			return this._kv[key];
		}
	}, {
		key: "invalidated",
		value: function invalidated(iso) {
			this._invalidated = true;
		}
	}, {
		key: "obsolete",
		value: function obsolete(iso) {}
	}, {
		key: "remove",
		value: function remove(key) {
			var iso = this._kv[key];
			var impl = iso && iso.__();

			if (impl) {
				impl.invalidate();
				impl.obsoleteTree();
				impl.blockFurtherUpdates(false);
			}
		}
	}, {
		key: "removeAll",
		value: function removeAll() {
			var kv = this._kv;

			for (var key in kv) {
				this.remove(key);
			}
		}
	}, {
		key: "set",
		value: function set(key, isoType) {
			var owner = this._owner;
			var prevIso = this._kv[key];
			var name = "<iso:" + owner.dotPath() + ":" + key + ">";

			// create and record the new isolated property
			var iso = this._nextKv[key] = new _IsolatedProperty2.default(isoType);

			iso.setKey(key);
			iso.setOwner(owner);
			iso.setStore(owner.store());

			// kill prevIso
			if (prevIso) {
				prevIso.__().blockFurtherUpdates(true);
			}
		}
	}, {
		key: "update",
		value: function update() {
			if (!this.invalidated && this._died.length === 0) {
				return;
			}

			var kv = this._kv;
			var nextKv = this._nextKv;
			var time = this._owner.__().time();
			var iso, impl;

			// resahdow live properties
			for (var name in kv) {
				iso = kv[name];
				impl = iso.__();

				if (impl.replaced() || !impl.isActive()) {
					impl.obsoleteTree();
					delete kv[name];
				} else if (!impl.isValid()) {
					impl.willShadow(false);

					(0, _reshadow2.default)(time, impl.nextState(), impl);
				}
			}

			// shadow the new iso properties (not replacements)
			for (var _name in nextKv) {
				var nextIso = nextKv[_name];
				var state = nextIso.initialState();
				var nextImpl = nextIso.shader().shadowProperty(time, _name, state);

				this._kv[_name] = nextIso;
			}

			this._nextKv = {};
			this._invalidated = false;
		}
	}, {
		key: "willUnshadow",
		value: function willUnshadow(iso) {}
	}]);

	return OwnerRecord;
}();