"use strict";

var _MapProperty = require("../MapProperty");

var _MapProperty2 = _interopRequireDefault(_MapProperty);

var _Store = require("../Store");

var _Store2 = _interopRequireDefault(_Store);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe("MapShadow", function () {
	var root, shadow, store;

	beforeEach(function () {
		root = new _MapProperty2.default();
		store = new _Store2.default(root, { a: 1, b: 2 });
		shadow = store._;
	});

	test("clear()", function () {
		shadow.clear();

		expect(Object.keys(shadow.$().nextState()).length).toBe(0);

		return store.waitThen().then(function () {
			expect(store.shadow.size).toBe(0);
		});
	});

	test("delete()", function () {
		shadow.delete("a");

		expect(Object.keys(shadow.$().nextState()).length).toBe(1);

		return store.waitThen().then(function () {
			expect(store.shadow.size).toBe(1);
		});
	});

	test("delete() bad key", function () {
		shadow.delete("badKey");

		expect(Object.keys(shadow.$().nextState()).length).toBe(2);

		return store.waitThen().then(function () {
			expect(store.shadow.size).toBe(2);
		});
	});

	test("get()", function () {
		expect(shadow.get("a")).toBe(1);
	});

	test("has()", function () {
		expect(shadow.has("a")).toBeTruthy;
		expect(shadow.has("b")).toBeTruthy;
		expect(shadow.has("badKey")).toBeFalsy;
	});

	test("keys()", function () {
		var expected = ["a", "b"];
		var found = [];

		var _iteratorNormalCompletion = true;
		var _didIteratorError = false;
		var _iteratorError = undefined;

		try {
			for (var _iterator = shadow.keysArray()[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
				var key = _step.value;

				found.push(key);
				expect(expected).toContain(key);
			}
		} catch (err) {
			_didIteratorError = true;
			_iteratorError = err;
		} finally {
			try {
				if (!_iteratorNormalCompletion && _iterator.return) {
					_iterator.return();
				}
			} finally {
				if (_didIteratorError) {
					throw _iteratorError;
				}
			}
		}

		expect(found).toHaveLength(expected.length);
	});

	test("keysArray()", function () {
		var expected = ["a", "b"];

		expect(shadow.keysArray()).toEqual(expect.arrayContaining(expected));
	});

	test("size", function () {
		expect(shadow.size).toBe(2);
	});

	test("values()", function () {
		var expected = [1, 2];
		var found = [];

		var _iteratorNormalCompletion2 = true;
		var _didIteratorError2 = false;
		var _iteratorError2 = undefined;

		try {
			for (var _iterator2 = shadow.values()[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
				var key = _step2.value;

				found.push(key);
				expect(expected).toContain(key);
			}
		} catch (err) {
			_didIteratorError2 = true;
			_iteratorError2 = err;
		} finally {
			try {
				if (!_iteratorNormalCompletion2 && _iterator2.return) {
					_iterator2.return();
				}
			} finally {
				if (_didIteratorError2) {
					throw _iteratorError2;
				}
			}
		}

		expect(found).toHaveLength(expected.length);
	});

	test("valuesArray()", function () {
		var expected = [1, 2];

		expect(shadow.valuesArray()).toEqual(expect.arrayContaining(expected));
	});

	test("dot access", function () {
		expect(shadow.a).toBe(1);
	});
});