"use strict";

var _IsolatedObjectShadowImpl = require("../IsolatedObjectShadowImpl");

var _IsolatedObjectShadowImpl2 = _interopRequireDefault(_IsolatedObjectShadowImpl);

var _MapProperty = require("../MapProperty");

var _MapProperty2 = _interopRequireDefault(_MapProperty);

var _Store = require("../Store");

var _Store2 = _interopRequireDefault(_Store);

var _StateType = require("../StateType");

var _StateType2 = _interopRequireDefault(_StateType);

var _LifecycleProperty = require("./LifecycleProperty");

var _LifecycleProperty2 = _interopRequireDefault(_LifecycleProperty);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe("isolated", function () {
	var first0, root, shadow, store;

	beforeEach(function () {
		var rootType = _MapProperty2.default.type.implementationClass(_IsolatedObjectShadowImpl2.default).managedType(_LifecycleProperty2.default.type);

		root = new _MapProperty2.default(rootType);
		store = new _Store2.default(root, {});
		shadow = store._;
		shadow.set("first", { a: 1, b: 2 });

		return store.waitThen().then(function () {
			shadow = store._;
			first0 = shadow.get("first");
		});
	});

	test("attribute assignment", function () {
		first0.a = 2;

		expect(shadow.$().isValid()).toBeFalsy();
		expect(first0.$().isValid()).toBeFalsy();
		first0.$$().checkPropCount(1, 1, 1, 0, 0);

		return store.waitThen().then(function () {
			var newShadow = store.shadow;
			var first = newShadow.get("first");

			expect(first.a).toBe(2);
			first.$$().checkPropCount(1, 1, 1, 1, 0);
		});
	});

	test("clear()", function () {
		shadow.clear();

		return store.waitThen().then(function () {
			var newShadow = store.shadow;
			var first = newShadow.get("first");

			expect(first).toBeUndefined();
			first0.$$().checkPropCount(1, 1, 0, 0, 1);
		});
	});

	test("has()", function () {
		expect(shadow.has("first")).toBeTruthy();
	});

	test("keysArray()", function () {
		expect(shadow.keysArray()).toEqual(expect.arrayContaining(["first"]));
	});

	test("remove()", function () {
		shadow.delete("first");

		return store.waitThen().then(function () {
			var newShadow = store.shadow;
			var first = newShadow.get("first");

			expect(first).toBeUndefined();
			first0.$$().checkPropCount(1, 1, 0, 0, 1);
		});
	});

	test("iso replaced", function () {
		first0.$$().checkPropCount(1, 1, 0, 0, 0);
		shadow.set("first", { z: 1 });
		first0.$$().checkPropCount(1, 1, 2, 0, 0);

		return store.waitThen().then(function () {
			var newShadow = store.shadow;
			var first = newShadow.get("first");

			expect(first.z).toBe(1);
			first0.$$().checkPropCount(1, 1, 2, 0, 1);
			first.$$().checkPropCount(1, 1, 0, 0, 0);
		});
	});

	test("set", function () {
		expect(shadow.size).toBe(1);
		expect(first0).toBeTruthy();
		expect(first0.a).toBe(1);

		expect(first0.__().root()).toBe(shadow.__());
		first0.$$().checkPropCount(1, 1, 0, 0, 0);
	});

	test("size()", function () {
		expect(shadow.size).toEqual(1);
	});

	test("delete isolated map", function () {
		// Initial state explicit to null so shadow.delete("iso") will not cause a property regeneration.
		var isoType = _LifecycleProperty2.default.type.implementationClass(_IsolatedObjectShadowImpl2.default).initialState(null).managedType(_LifecycleProperty2.default.type);
		var rootType = _MapProperty2.default.type.properties({ iso: isoType });
		var first0, iso, shadow;

		// explicitly set 'iso' initial state so something to delete
		root = new _MapProperty2.default(rootType);
		store = new _Store2.default(root, { iso: {} });

		store._.iso.set("first", { a: 1, b: 2 });

		return store.waitThen().then(function () {
			shadow = store._;
			iso = shadow.iso;
			first0 = iso.get("first");

			first0.$$().checkPropCount(1, 1, 0, 0, 0);
			iso.$$().checkPropCount(1, 1, 0, 1, 0);

			shadow.delete("iso");

			first0.$$().checkPropCount(1, 1, 2, 0, 0);
			iso.$$().checkPropCount(1, 1, 1, 1, 0);

			return store.waitThen();
		}).then(function () {
			first0.$$().checkPropCount(1, 1, 2, 0, 1);
			iso.$$().checkPropCount(1, 1, 1, 1, 1);
		});
	});
});