"use strict";

var _ObjectProperty = require("../ObjectProperty");

var _ObjectProperty2 = _interopRequireDefault(_ObjectProperty);

var _Store = require("../Store");

var _Store2 = _interopRequireDefault(_Store);

var _StateType = require("../StateType");

var _StateType2 = _interopRequireDefault(_StateType);

var _LifecycleProperty = require("./LifecycleProperty");

var _LifecycleProperty2 = _interopRequireDefault(_LifecycleProperty);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe("life-cycle", function () {
	var root, shadow, store;

	beforeEach(function () {
		var rootType = _StateType2.default.create(_ObjectProperty2.default).elementType(_LifecycleProperty2.default.type);
		var state = {
			first: { a: 1, b: 2 },
			second: { a: 1, b: 2 }
		};

		root = new _LifecycleProperty2.default(rootType);
		store = new _Store2.default(root, state);
		shadow = store._;
	});

	test("willShadow", function () {
		var firstProp = store.shadow.first.$$();

		root.checkPropCount(1, 1, 0, 0, 0);
		firstProp.checkPropCount(1, 1, 0, 0, 0);
	});

	test("single update", function () {
		shadow.first.a = 2;

		return store.waitThen().then(function () {
			var firstProp = store.shadow.first.$$();

			expect(store.shadow.first.a).toBe(2);
			root.checkPropCount(1, 1, 1, 1, 0);
			firstProp.checkPropCount(1, 1, 1, 1, 0);
		});
	});

	test("multiple updates same leaf", function () {
		shadow.first.a = 2;
		shadow.first.a = 3;

		return store.waitThen().then(function () {
			var firstProp = store.shadow.first.$$();

			expect(store.shadow.first.a).toBe(3);
			root.checkPropCount(1, 1, 1, 1, 0);
			firstProp.checkPropCount(1, 1, 1, 1, 0);
		});
	});

	test("shadow.first = { z: 1 }", function () {
		var first0 = store.shadow.first.$$();
		var first0Impl = first0.__();

		shadow.first = { z: 1 };
		shadow.first = { z: 2 };

		expect(first0Impl.isActive()).toBeTruthy();
		expect(first0Impl.isValid()).toBeFalsy();
		expect(first0Impl.replaced()).toBeTruthy();

		return store.waitThen().then(function () {
			var firstProp = store.shadow.first.$$();
			var firstImpl = firstProp.__();

			expect(first0Impl.isActive()).toBeFalsy();
			expect(first0Impl.isValid()).toBeFalsy();
			expect(first0Impl.replaced()).toBeTruthy();

			expect(firstImpl.isActive()).toBeTruthy();
			expect(firstImpl.isValid()).toBeTruthy();
			expect(firstImpl.replaced()).toBeFalsy();

			expect(firstProp._().z).toBe(2);
			expect(first0 === firstProp).toBeFalsy();
			root.checkPropCount(1, 1, 1, 1, 0);
			first0.checkPropCount(1, 1, 2, 0, 1);
			firstProp.checkPropCount(1, 1, 0, 0, 0);
		});
	});
});